
# q-expansion of newform 5929.2.a.h, downloaded from the LMFDB on 06 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5929
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[2], [1], [-1], [0], [0], [4], [-2], [0], [-1], [0], [-7], [3], [-8], [6], [-8], [-6], [-5], [12], [-7], [-3], [4], [10], [-6], [-15], [7], [2], [16], [-18], [-10], [9], [-8], [-18], [-7], [10], [10], [-2], [7], [4], [-12], [-6], [-15], [-7], [17], [-4], [2], [0], [-12], [-19], [18], [-15], [-24], [30], [-8], [23], [2], [-14], [-10], [-28], [2], [18], [4], [24], [8], [-12], [1], [13], [7], [22], [-28], [30], [21], [20], [17], [26], [-5], [1], [-15], [2], [2], [-30], [-20], [22], [18], [11], [40], [-11], [35], [12], [12], [-11], [27], [20], [23], [8], [20], [-26], [-15], [3], [-16], [8], [-8], [2], [4], [0], [28], [-33], [-28], [44], [40], [2], [-22], [16], [18], [25], [7], [-33], [-29], [7], [-41], [-10], [-37], [-14], [-42], [-16], [-17], [-2], [-25], [-15], [-3], [-36], [-50], [-4], [-23], [-22], [12], [20], [6], [-32], [-53], [0], [-38], [-22], [39], [52], [-25], [5], [14], [8], [15], [24], [12], [43], [4], [-22], [-12], [12], [-10], [30], [8], [42], [-27], [-34], [32], [-47], [-27], [-39], [-8], [38], [10], [-39], [10], [-22], [-32], [16], [-5], [55], [-2], [-13], [44], [-20], [8], [-58], [-51], [-42], [-51], [30], [-48], [-24], [-50], [2], [31], [-34], [-3], [18], [12], [21], [-2], [-41], [-42], [14], [-60], [-18], [-18], [40], [25], [47], [15], [36], [0], [8], [-48], [-27], [39], [-28], [-30], [-47], [-68], [12], [-72], [39], [68], [60], [-15], [29], [-12], [70], [-54], [0], [28], [-52], [-71], [-20], [-22], [-32], [-49], [58], [15], [36], [55], [-37], [-41], [32], [36], [-15], [-56], [-60], [52], [28], [30], [-34], [32], [2], [33], [10], [-6], [20], [22], [78], [-33], [-2], [-4], [-48], [50], [-6], [42], [-40], [-45], [3], [46], [6], [-17], [-57], [-34], [40], [8], [-59], [-57], [-10], [-52], [12], [-56], [-43], [-52], [62], [28], [-3], [-6], [-18], [35], [70], [77], [52], [36], [-18], [-54], [-40], [23], [-79], [30], [-22], [-66], [72], [20], [4], [-13], [-17], [63], [-45], [60], [-84], [24], [70], [18], [89], [-48], [-10], [-35], [38], [86], [-20], [-68], [73], [-58], [-91], [26], [13], [-45], [1], [-48], [-4], [-22], [78], [-70], [-56], [48], [-93], [25], [4], [-7], [38], [29], [-57], [60], [-13], [59], [-10], [-67], [-37], [-48], [57], [-28], [3], [-18], [-36], [50], [54], [75], [62], [22], [-31], [82], [42], [3], [50], [-3], [-11], [48], [-14], [-72], [57], [0], [-34], [20], [-98], [13], [-20], [58], [14], [-30], [18], [-22], [39], [-38], [-38], [40], [-39], [72], [7], [16], [-23], [5], [41], [-55], [17], [87], [56], [-70], [-30], [-68], [58], [50], [49], [-48], [42], [-20], [42], [-42], [-52], [-44], [25], [-6], [62], [4], [-2], [82], [63], [40], [57], [-38], [54], [-25], [88], [-72], [-50], [86], [-3], [-81], [-70], [53], [-80]]
